/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.secure;

import org.wso2.registry.ActionConstants;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.jdbc.DatabaseConstants;
import org.wso2.registry.jdbc.ConnectionFactory;
import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerException;
import org.wso2.usermanager.readwrite.DefaultRealm;
import org.wso2.usermanager.readwrite.DefaultRealmConfig;

import javax.sql.DataSource;

public class RegistryRealm {

    public static Realm createInMemoryRegistryRealm() throws RegistryException {
        new HSQLDBInitializer().createHSQLTables();
        try {
            DefaultRealm realm = new DefaultRealm();
            DefaultRealmConfig config = (DefaultRealmConfig)realm.getRealmConfiguration();
            config.setConnectionURL(UserManagerConstants.HSQL_DB_URL);
            config.setDriverName(DatabaseConstants.HSQL_DRIVER_NAME);
            realm.init(config);
            // define built in roles
            populateRoles(realm);
            return realm;

        } catch (Exception e) {
            throw new RegistryException(e.getMessage());
        }
    }

    public static Realm createDataSourceAwareRegistryRealm() throws RegistryException {
        new HSQLDBInitializer().createHSQLTables();
         try {
             DefaultRealm realm = new DefaultRealm();
             DefaultRealmConfig config = (DefaultRealmConfig)realm.getRealmConfiguration();
             DataSource dataSource =  new ConnectionFactory().getDataSource();
             config.setDataSource(dataSource);
             realm.init(config);
             // define built in roles
            populateRoles(realm);
            return realm;

        } catch (Exception e) {
            e.printStackTrace();
            throw new RegistryException(e.getMessage());
        }
    }



    private static void populateRoles(DefaultRealm realm) throws UserManagerException {
        String[] roles = realm.getAllRoleNames();

        if (!containsString(RegistryConstants.ADMIN_ROLE, roles)) {

            realm.addRole(RegistryConstants.ADMIN_ROLE);

            realm.authorizeRole(RegistryConstants.ADMIN_ROLE, RegistryConstants.ROOT_PATH, ActionConstants.GET);
            realm.authorizeRole(RegistryConstants.ADMIN_ROLE, RegistryConstants.ROOT_PATH, ActionConstants.PUT);
            realm.authorizeRole(RegistryConstants.ADMIN_ROLE, RegistryConstants.ROOT_PATH, ActionConstants.DELETE);
            realm.authorizeRole(RegistryConstants.ADMIN_ROLE, RegistryConstants.ROOT_PATH, ActionConstants.AUTHORIZE);

            // authorizations for registry level actions are set in the root path
            realm.authorizeRole(RegistryConstants.ADMIN_ROLE, RegistryConstants.ROOT_PATH, ActionConstants.ADD_USER);
            realm.authorizeRole(RegistryConstants.ADMIN_ROLE, RegistryConstants.ROOT_PATH, ActionConstants.REMOVE_USER);
            realm.authorizeRole(RegistryConstants.ADMIN_ROLE, RegistryConstants.ROOT_PATH, ActionConstants.ADD_ROLE);
            realm.authorizeRole(RegistryConstants.ADMIN_ROLE, RegistryConstants.ROOT_PATH, ActionConstants.REMOVE_ROLE);
        }

        if (!containsString(RegistryConstants.GUESTS_ROLE, roles)) {
            realm.addRole(RegistryConstants.GUESTS_ROLE);
        }

        // define built in users

        String[] users = realm.getAllUserNames();

        if (!containsString(RegistryConstants.ADMIN_USER, users)) {

            realm.addUser(RegistryConstants.ADMIN_USER, "admin");

            realm.authorizeUser(RegistryConstants.ADMIN_USER, RegistryConstants.ROOT_PATH, ActionConstants.GET);
            realm.authorizeUser(RegistryConstants.ADMIN_USER, RegistryConstants.ROOT_PATH, ActionConstants.PUT);
            realm.authorizeUser(RegistryConstants.ADMIN_USER, RegistryConstants.ROOT_PATH, ActionConstants.DELETE);
            realm.authorizeUser(RegistryConstants.ADMIN_USER, RegistryConstants.ROOT_PATH, ActionConstants.AUTHORIZE);

            // authorizations for registry level actions are set in the root path
            realm.authorizeUser(RegistryConstants.ADMIN_USER, RegistryConstants.ROOT_PATH, ActionConstants.ADD_USER);
            realm.authorizeUser(RegistryConstants.ADMIN_USER, RegistryConstants.ROOT_PATH, ActionConstants.REMOVE_USER);
            realm.authorizeUser(RegistryConstants.ADMIN_ROLE, RegistryConstants.ROOT_PATH, ActionConstants.ADD_ROLE);
            realm.authorizeUser(RegistryConstants.ADMIN_ROLE, RegistryConstants.ROOT_PATH, ActionConstants.REMOVE_ROLE);
        }

        if (!containsString(RegistryConstants.ANONYMOUS_USER, users)) {
            realm.addUser(RegistryConstants.ANONYMOUS_USER, "guest");
        }
    }

    private static boolean containsString(String value, String[] array) {
        for (int i = 0; i < array.length; i++) {
            if (value.equals(array[i])) {
                return true;
            }
        }
        return false;
    }
}
